@@ -23,7 +23,7 @@ from zope.location import Location
 from pyams_batching.interfaces import IBatch
 from pyams_table.column import NoneCell
 from pyams_table.interfaces import IBatchProvider, IColumn, INoneCell, ISequenceTable, ITable, \
-    IValues
+    IValues, TableRowUpdatedEvent, TableUpdatedEvent
 from pyams_utils.factory import get_object_factory
 
 
@@ -82,6 +82,10 @@ class Table(Location):
     batch_size = 50
     start_batching_at = 50
 
+    # events attributes
+    handle_table_events = True
+    handle_row_events = True
+
     def __init__(self, context, request):
         self.context = context
         self.request = request
@@ -94,6 +98,7 @@ class Table(Location):
         self.column_by_index = {}
         self.columns = None
         self.rows = []
+        self.rows_state = {}
         self.selected_items = []
 
     def init_columns(self):
@@ -127,21 +132,19 @@ class Table(Location):
             sort_id = column.id.rsplit("-", 1)[-1]
 
             if int(sort_id) == int(current_sort_id):
-                klass = self.css_class_sorted_on + " " + self.sort_order
-                if css_class:
-                    css_class += " " + klass
-                else:
-                    css_class = klass
+                css_class = f"{css_class if css_class else ''} " \
+                            f"{self.css_class_sorted_on} {self.sort_order}"
         return css_class
 
-    def get_css_class(self, element, css_class=None):
+    def get_css_class(self, element, css_class=None, *args, **kwargs):
         """Add CSS class based on HTML tag, make a `class=` attribute"""
+        if callable(css_class):
+            css_class = css_class(*args, **kwargs)
         klass = self.css_classes.get(element)
-        if klass and css_class:
-            klass = "%s %s" % (css_class, klass)
-        elif css_class:
-            klass = css_class
-        return " class=%s" % quoteattr(klass) if klass else ""
+        if callable(klass):
+            klass = klass(*args, **kwargs)
+        klass = f"{css_class if css_class else ''} {klass if klass else ''}".strip()
+        return f' class={quoteattr(klass)}' if klass else ""
 
     # setup
 
@@ -195,6 +198,8 @@ class Table(Location):
                 raise ValueError("Colspan for column '%s' is larger than the table." % col)
 
             append((item, col, colspan))
+        if self.handle_row_events:
+            self.request.registry.notify(TableRowUpdatedEvent(self, item))
         return cols
 
     def setup_rows(self):
@@ -310,10 +315,9 @@ class Table(Location):
     def render_row(self, row, css_class=None):
         """Render given row"""
         is_selected = self.is_selected_row(row)
-        if is_selected and self.css_class_selected and css_class:
-            css_class = "%s %s" % (self.css_class_selected, css_class)
-        elif is_selected and self.css_class_selected:
-            css_class = self.css_class_selected
+        if is_selected:
+            css_class = f"{self.css_class_selected if self.css_class_selected else ''} " \
+                        f"{css_class if css_class else ''}".strip()
         css_class = self.get_css_class("tr", css_class)
         cells = [self.render_cell(item, col, colspan) for item, col, colspan in row]
         return '\n    <tr%s>%s\n    </tr>' % (css_class, ''.join(cells))
@@ -327,6 +331,8 @@ class Table(Location):
         if INoneCell.providedBy(column):
             return ""
         css_class = column.css_classes.get("td")
+        if callable(css_class):
+            css_class = css_class(item, column)
         css_class = self.get_css_highlight_class(column, item, css_class)
         css_class = self.get_css_sort_class(column, css_class)
         css_class = self.get_css_class("td", css_class)
@@ -375,6 +381,10 @@ class Table(Location):
 
         self.update_batch()
 
+        # notify table update
+        if self.handle_table_events:
+            self.request.registry.notify(TableUpdatedEvent(self))
+
     def render(self):
         """Render table contents"""
         # allow to use a template for rendering the table, this will allow
